@@ -1,7 +1,7 @@
 /*
     JackEngine.h
 
-    Copyright 2009-2010, Alan Calvert
+    Copyright 2009-2011, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -39,16 +39,12 @@ class JackEngine : public MusicIO
     public:
         JackEngine();
         ~JackEngine() { Close(); };
-
         bool isConnected(void) { return (NULL != jackClient); };
         bool connectServer(string server);
-        bool openAudio(WavRecord *recorder);
-        bool openMidi(WavRecord *recorder);
+        bool openAudio(void);
+        bool openMidi(void);
         bool Start(void);
         void Close(void);
-        #if defined(JACK_SESSION)
-            bool jacksessionReply(string cmdline);
-        #endif
         unsigned int getSamplerate(void) { return audio.jackSamplerate; };
         int getBuffersize(void) { return audio.jackNframes; };
         string clientName(void);
@@ -56,24 +52,28 @@ class JackEngine : public MusicIO
 
     private:
         bool openJackClient(string server);
-        bool new_openJackClient(string server);
         bool connectJackPorts(void);
         bool processAudio(jack_nframes_t nframes);
         bool processMidi(jack_nframes_t nframes);
+        bool latencyPrep(void);
         int processCallback(jack_nframes_t nframes);
         static int _processCallback(jack_nframes_t nframes, void *arg);
         static void *_midiThread(void *arg);
         void *midiThread(void);
-        void midiCleanup(void);
-        static void _midiCleanup(void *arg);
         static void _errorCallback(const char *msg);
         static int _xrunCallback(void *arg);
 
-        #if defined(JACK_SESSION)
+
+#if defined(JACK_SESSION)
             static void _jsessionCallback(jack_session_event_t *event, void *arg);
             void jsessionCallback(jack_session_event_t *event);
             jack_session_event_t *lastevent;
-        #endif
+#endif
+
+#if defined(JACK_LATENCY)
+            static void _latencyCallback(jack_latency_callback_mode_t mode, void *arg);
+            void latencyCallback(jack_latency_callback_mode_t mode);
+#endif
 
         jack_client_t      *jackClient;
         struct {
@@ -87,15 +87,14 @@ class JackEngine : public MusicIO
             jack_port_t*       port;
             jack_ringbuffer_t *ringBuf;
             pthread_t          pThread;
-            string             semName;
-            sem_t             *eventsUp;
         } midi;
 
+        sem_t midiSem;
+
         struct midi_event {
             jack_nframes_t time;
             char data[4]; // all events of interest are <= 4bytes
         };
-
 };
 
 #endif
